@@ -414,11 +414,7 @@ create_option_frame(struct file_prm *prm)
 
   frame = gtk_frame_new("option");
 
-#if GTK_CHECK_VERSION(3, 0, 0)
   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
-#else
-  vbox = gtk_vbox_new(FALSE, 4);
-#endif
 
   w = gtk_check_button_new_with_mnemonic("_Mix");
   gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 2);
@@ -461,11 +457,7 @@ create_geometry_frame(struct file_prm *prm)
 
   frame = gtk_frame_new("geometry");
 
-#if GTK_CHECK_VERSION(3, 4, 0)
   table = gtk_grid_new();
-#else
-  table = gtk_table_new(1, 2, FALSE);
-#endif
 
   j = 0;
   w = create_spin_button(POS_MIN, POS_MAX, POS_INC, prm->posx / 100.0, 2);
@@ -627,11 +619,7 @@ create_file_frame(struct file_prm *prm)
   frame = gtk_frame_new(NULL);
   gtk_container_add(GTK_CONTAINER(frame), swin);
 
-#if GTK_CHECK_VERSION(3, 0, 0)
   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
-#else
-  hbox = gtk_hbox_new(FALSE, 4);
-#endif
   gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 4);
 
   prm->files = tview;
@@ -644,11 +632,7 @@ create_control(GtkWidget *box, struct file_prm *prm)
 {
   GtkWidget *w, *hbox;
 
-#if GTK_CHECK_VERSION(3, 0, 0)
   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
-#else
-  hbox = gtk_hbox_new(FALSE, 4);
-#endif
   w = create_option_frame(prm);
   gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 4);
 
@@ -717,11 +701,7 @@ main(int argc, char **argv)
   struct file_prm prm;
   const char *data_file;
 
-#if GTK_CHECK_VERSION(2, 24, 0)
   setlocale(LC_ALL, "");
-#else
-  gtk_set_locale();
-#endif
   gtk_init(&argc, &argv);
 
   prm.posx = POS_X;
