@@ -10,23 +10,28 @@ then
     exit
 fi
 
+MO_FILES="gtk30.mo ngraph-gtk.mo"
+
 VERSION=`basename $ARCHIVE '.tar.gz' | sed -e 's/ngraph-gtk-//'`
 
 BINFILES="libatk-1.0-0.dll libbz2-1.dll libcairo-2.dll
 libcairo-gobject-2.dll libepoxy-0.dll libexpat-1.dll libffi-6.dll
 libfontconfig-1.dll libfreetype-6.dll libgdk_pixbuf-2.0-0.dll
 libgdk-3-0.dll libgio-2.0-0.dll libglib-2.0-0.dll libgmodule-2.0-0.dll
-libgobject-2.0-0.dll libgraphite2.dll libgsl-19.dll libgslcblas-0.dll
+libgobject-2.0-0.dll libgraphite2.dll libgsl-23.dll libgslcblas-0.dll
 libgtk-3-0.dll libharfbuzz-0.dll libiconv-2.dll libintl-8.dll
 libobjc-4.dll libp11-kit-0.dll libpango-1.0-0.dll
-libpangocairo-1.0-0.dll libpangoft2-1.0-0.dll libpangowin32-1.0-0.dll
-libpcre-1.dll libpixman-1-0.dll libpng16-16.dll libreadline7.dll
-libstdc++-6.dll libtermcap-0.dll libwinpthread-1.dll zlib1.dll
-libcroco-0.6-3.dll liblzma-5.dll librsvg-2-2.dll libxml2-2.dll
-libngraph-0.dll ngraph.exe ngp2"
+libgtksourceview-3.0-1.dll libpangocairo-1.0-0.dll
+libpangoft2-1.0-0.dll libpangowin32-1.0-0.dll libpcre-1.dll
+libpixman-1-0.dll libpng16-16.dll libstdc++-6.dll libtermcap-0.dll
+libwinpthread-1.dll zlib1.dll libcroco-0.6-3.dll liblzma-5.dll
+librsvg-2-2.dll libxml2-2.dll libngraph-0.dll ngraph.exe ngp2"
 
-BINFILES64="libgcc_s_seh-1.dll"
-BINFILES32="libgcc_s_dw2-1.dll"
+READLINE7="libreadline7.dll"
+READLINE6="libreadline6.dll"
+
+BINFILES64="libgcc_s_seh-1.dll gspawn-win64-helper-console.exe"
+BINFILES32="libgcc_s_dw2-1.dll gspawn-win32-helper-console.exe"
 
 make_zip() {
     echo create win$1 archive.
@@ -59,6 +64,13 @@ make_zip() {
 			cp $win_path/$subdir/$i $PKG_DIR/$subdir/
 		    done
 		fi
+		if [ -e "$win_path/$subdir/$READLINE6" ]
+		then
+		    cp $win_path/$subdir/$READLINE6 $PKG_DIR/$subdir/
+		elif [ -e "$win_path/$subdir/$READLINE7" ]
+		then
+		    cp $win_path/$subdir/$READLINE7 $PKG_DIR/$subdir/
+		fi
 		;;
 	    etc)
 		for i in fonts gtk-3.0 ngraph-gtk
@@ -73,10 +85,31 @@ make_zip() {
 		done
 		;;
 	    share)
-		for i in GConf glib-2.0 icons locale themes ngraph-gtk pixmaps
+		for i in GConf glib-2.0 gtksourceview-3.0 icons themes ngraph-gtk
 		do
 		    cp -r $win_path/$subdir/$i $PKG_DIR/$subdir/
 		done
+
+		locale_dir="$PKG_DIR/$subdir/locale"
+		mkdir "$locale_dir"
+		for i in $win_path/$subdir/locale/*
+		do
+		    if [ -d "$i" ]
+		    then
+			modir=`basename "$i"`/LC_MESSAGES
+			mkdir -p "$locale_dir/$modir"
+			for mo in $MO_FILES
+			do
+			    mofile="$i"/LC_MESSAGES/$mo
+			    if [ -f $mofile ]
+			    then
+				cp $mofile $locale_dir/$modir/
+			    fi
+			done
+		    else
+			cp "$i" $PKG_DIR/$subdir/locale/
+		    fi
+		done
 		mkdir $PKG_DIR/$subdir/doc
 		cp -r $win_path/$subdir/doc/ngraph-gtk $PKG_DIR/$subdir/doc
 		;;
