@@ -28,6 +28,14 @@
 #define _WIN32_WINNT 0x0500
 #endif
 #include <windows.h>
+#else
+#define WINDOWS 0
+#endif
+
+#ifdef __APPLE__
+#define OSX 1
+#else
+#define OSX 0
 #endif
 
 #ifndef CCNAME
@@ -40,8 +48,6 @@
 
 #define COMPILER_NAME (CCNAME " " __VERSION__)
 
-#define USE_MEM_PROFILE 0
-
 #define MARK_TYPE_NUM 90
 
 int printfconsole(const char *fmt,...);
