@@ -89,15 +89,6 @@ file_select(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointe
   }
 }
 
-#if GTK_CHECK_VERSION(3, 0, 0) && ! GTK_CHECK_VERSION(3, 2, 0)
-static gboolean
-cell_focus_out(GtkWidget *widget, GdkEvent *event, gpointer user_data)
-{
-  menu_lock(FALSE);
-  return FALSE;
-}
-#endif
-
 static void
 select_enum(GtkComboBox *w, gpointer user_data)
 {
@@ -120,6 +111,7 @@ select_enum(GtkComboBox *w, gpointer user_data)
   if (j < 0 || j == val)
     return;
 
+  menu_save_undo_single(UNDO_TYPE_EDIT, d->obj->name);
   if (putobj(d->obj, list->name, sel, &j) >= 0) {
     d->select = sel;
   }
@@ -220,10 +212,6 @@ start_editing(GtkCellRenderer *renderer, GtkCellEditable *editable, gchar *path,
 	  g_free(valstr);
 	}
       }
-#if GTK_CHECK_VERSION(3, 0, 0) && ! GTK_CHECK_VERSION(3, 2, 0)
-      /* this code may need to avoid a bug of GTK+ 3.0 */
-      g_signal_connect(editable, "focus-out-event", G_CALLBACK(cell_focus_out), d);
-#endif
     }
     break;
   case G_TYPE_DOUBLE:
@@ -298,7 +286,7 @@ enum_cb(GtkCellRenderer *cell_renderer, gchar *path, gchar *str, gpointer user_d
   if (str == NULL || d->select < 0)
     return;
 
-  d->update(d, FALSE);
+  d->update(d, FALSE, TRUE);
   set_graph_modified();
 }
 
@@ -810,6 +798,15 @@ cb_destroy(GtkWidget *w, gpointer user_data)
   }
 }
 
+void
+update_viewer(struct obj_list_data *d)
+{
+  char *objects[2];
+  objects[0] = d->obj->name;
+  objects[1] = NULL;
+  ViewerWinUpdate(objects);
+}
+
 static void
 obj_copy(struct objlist *obj, int dest, int src)
 {
@@ -821,7 +818,7 @@ obj_copy(struct objlist *obj, int dest, int src)
 static void
 copy(struct obj_list_data *d)
 {
-  int sel, id, num;
+  int sel, id, num, undo;
 
   if (Menulock || Globallock)
     return;
@@ -832,13 +829,16 @@ copy(struct obj_list_data *d)
   num = chkobjlastinst(d->obj);
 
   if (sel >= 0 && sel <= num) {
+    undo = menu_save_undo_single(UNDO_TYPE_COPY, d->obj->name);
     id = newobj(d->obj);
-    if (id >= 0) {
-      obj_copy(d->obj, id, sel);
-      set_graph_modified();
-      d->select = id;
-      d->update(d, FALSE);
+    if (id < 0) {
+      menu_delete_undo(undo);
+      return;
     }
+    obj_copy(d->obj, id, sel);
+    set_graph_modified();
+    d->select = id;
+    d->update(d, FALSE, TRUE);
   }
 }
 
@@ -859,6 +859,7 @@ delete(struct obj_list_data *d)
 
   UnFocus();
 
+  menu_save_undo_single(UNDO_TYPE_DELETE, d->obj->name);
   if (d->delete) {
     d->delete(d, sel);
   } else {
@@ -876,7 +877,7 @@ delete(struct obj_list_data *d)
     d->select = sel;
     update = FALSE;
   }
-  d->update(d, update);
+  d->update(d, update, TRUE);
   set_graph_modified();
 }
 
@@ -896,9 +897,10 @@ move_top(struct obj_list_data *d)
 
   UnFocus();
 
+  menu_save_undo_single(UNDO_TYPE_ORDER, d->obj->name);
   movetopobj(d->obj, sel);
   d->select = 0;
-  d->update(d, FALSE);
+  d->update(d, FALSE, TRUE);
   set_graph_modified();
 }
 
@@ -918,9 +920,10 @@ move_last(struct obj_list_data *d)
     return;
   }
 
+  menu_save_undo_single(UNDO_TYPE_ORDER, d->obj->name);
   movelastobj(d->obj, sel);
   d->select = num;
-  d->update(d, FALSE);
+  d->update(d, FALSE, TRUE);
   set_graph_modified();
 }
 
@@ -937,9 +940,10 @@ move_up(struct obj_list_data *d)
   sel = list_store_get_selected_int(GTK_WIDGET(d->text), COL_ID);
   num = chkobjlastinst(d->obj);
   if ((sel >= 1) && (sel <= num)) {
+    menu_save_undo_single(UNDO_TYPE_ORDER, d->obj->name);
     moveupobj(d->obj, sel);
     d->select = sel - 1;
-    d->update(d, FALSE);
+    d->update(d, FALSE, TRUE);
     set_graph_modified();
   }
 }
@@ -957,9 +961,10 @@ move_down(struct obj_list_data *d)
   sel = list_store_get_selected_int(GTK_WIDGET(d->text), COL_ID);
   num = chkobjlastinst(d->obj);
   if ((sel >= 0) && (sel < num)) {
+    menu_save_undo_single(UNDO_TYPE_ORDER, d->obj->name);
     movedownobj(d->obj, sel);
     d->select = sel + 1;
-    d->update(d, FALSE);
+    d->update(d, FALSE, TRUE);
     set_graph_modified();
   }
 }
@@ -967,7 +972,7 @@ move_down(struct obj_list_data *d)
 static void
 update(struct obj_list_data *d)
 {
-  int sel, ret, num;
+  int sel, ret, num, undo;
   GtkWidget *parent;
 
   if (Menulock || Globallock)
@@ -985,16 +990,29 @@ update(struct obj_list_data *d)
 
   d->setup_dialog(d, sel, -1);
   d->select = sel;
-  if ((ret = DialogExecute(parent, d->dialog)) == IDDELETE) {
+  if (d->undo_save) {
+    undo = d->undo_save(UNDO_TYPE_EDIT);
+  } else {
+    undo = menu_save_undo_single(UNDO_TYPE_EDIT, d->obj->name);
+  }
+  ret = DialogExecute(parent, d->dialog);
+  set_graph_modified();
+  switch (ret) {
+  case IDCANCEL:
+    menu_undo_internal(undo);
+    break;
+  case IDDELETE:
     if (d->delete) {
       d->delete(d, sel);
     } else {
       delobj(d->obj, sel);
     }
     d->select = -1;
-    set_graph_modified();
+    d->update(d, FALSE, DRAW_REDRAW);
+    break;
+  default:
+    d->update(d, FALSE, DRAW_NOTIFY);
   }
-  d->update(d, FALSE);
 }
 
 static void
@@ -1025,6 +1043,7 @@ toggle_boolean(struct obj_list_data *d, char *field, int sel)
     return;
   }
 
+  menu_save_undo_single(UNDO_TYPE_EDIT, d->obj->name);
   getobj(d->obj, field, sel, 0, NULL, &v1);
   v1 = ! v1;
   if (putobj(d->obj, field, sel, &v1) < 0) {
@@ -1032,14 +1051,14 @@ toggle_boolean(struct obj_list_data *d, char *field, int sel)
   }
 
   d->select = sel;
-  d->update(d, FALSE);
+  d->update(d, FALSE, TRUE);
   set_graph_modified();
 }
 
 static void
 modify_numeric(struct obj_list_data *d, char *field, int val)
 {
-  int sel, v1, v2, num;
+  int sel, v1, v2, num, undo;
 
   if (Menulock || Globallock)
     return;
@@ -1050,8 +1069,8 @@ modify_numeric(struct obj_list_data *d, char *field, int val)
     return;
   }
 
+  undo = menu_save_undo_single(UNDO_TYPE_EDIT, d->obj->name);
   getobj(d->obj, field, sel, 0, NULL, &v1);
-
   if (putobj(d->obj, field, sel, &val) < 0) {
     return;
   }
@@ -1059,15 +1078,17 @@ modify_numeric(struct obj_list_data *d, char *field, int val)
   getobj(d->obj, field, sel, 0, NULL, &v2);
   if (v1 != v2) {
     d->select = sel;
-    d->update(d, FALSE);
+    d->update(d, FALSE, TRUE);
     set_graph_modified();
+  } else {
+    menu_delete_undo(undo);
   }
 }
 
 static void
 modify_string(struct obj_list_data *d, char *field, char *str)
 {
-  int sel, num;
+  int sel, num, undo;
 
   if (Menulock || Globallock)
     return;
@@ -1078,11 +1099,14 @@ modify_string(struct obj_list_data *d, char *field, char *str)
     return;
   }
 
-  if (chk_sputobjfield(d->obj, sel, field, str))
+  undo = menu_save_undo_single(UNDO_TYPE_EDIT, d->obj->name);
+  if (chk_sputobjfield(d->obj, sel, field, str)) {
+    menu_delete_undo(undo);
     return;
+  }
 
   d->select = sel;
-  d->update(d, FALSE);
+  d->update(d, FALSE, TRUE);
 }
 
 static void
@@ -1102,11 +1126,12 @@ hidden(struct obj_list_data *d)
 
   UnFocus();
 
+  menu_save_undo_single(UNDO_TYPE_EDIT, d->obj->name);
   getobj(d->obj, "hidden", sel, 0, NULL, &hidden);
   hidden = hidden ? FALSE : TRUE;
   putobj(d->obj, "hidden", sel, &hidden);
   d->select = sel;
-  d->update(d, FALSE);
+  d->update(d, FALSE, TRUE);
   set_graph_modified();
 }
 
@@ -1129,7 +1154,7 @@ set_hidden_state(struct obj_list_data *d, int hide)
   if (hidden != hide) {
     putobj(d->obj, "hidden", sel, &hide);
     d->select = sel;
-    d->update(d, FALSE);
+    d->update(d, FALSE, TRUE);
     set_graph_modified();
   }
 }
@@ -1152,6 +1177,12 @@ static void
 do_popup(GdkEventButton *event, struct obj_list_data *d)
 {
 #if GTK_CHECK_VERSION(3, 22, 0)
+  if (d->parent->type == TypeFileWin ||
+      d->parent->type == TypeAxisWin ||
+      d->parent->type == TypeMergeWin ||
+      d->parent->type == TypeLegendWin) {
+    d->select = list_store_get_selected_int(GTK_WIDGET(d->text), COL_ID);
+  }
   gtk_menu_popup_at_pointer(GTK_MENU(d->popup), ((GdkEvent *)event));
 #else
   int button, event_time;
@@ -1340,7 +1371,7 @@ ev_sub_win_key_down(GtkWidget *w, GdkEvent *event, gpointer user_data)
   return FALSE;
 }
 
-#ifdef WINDOWS
+#if WINDOWS
 #include <gdk/gdkwin32.h>
 
 static void
@@ -1362,7 +1393,7 @@ swin_realized(GtkWidget *widget, gpointer user_data)
 
   ptr = (struct obj_list_data *) user_data;
 
-  ptr->update(ptr, TRUE);
+  ptr->update(ptr, TRUE, TRUE);
 }
 
 static GtkWidget *
@@ -1379,14 +1410,7 @@ sub_window_create(struct SubWin *d, const char *title, GtkWidget *swin, const ch
   d->Win = dlg;
 
   if (iconfile) {
-#ifdef WINDOWS
-    char *str;
-    str = g_strdup_printf("%s%s", PIXMAPDIR, iconfile);
-    icon = gdk_pixbuf_new_from_file(str, NULL);
-    g_free(str);
-#else
-    icon = gdk_pixbuf_new_from_file(iconfile, NULL);
-#endif
+    icon = gdk_pixbuf_new_from_resource(iconfile, NULL);
 
     if (iconfile2) {
       GList *tmp, *list = NULL;
@@ -1395,13 +1419,7 @@ sub_window_create(struct SubWin *d, const char *title, GtkWidget *swin, const ch
 	list = g_list_append(list, icon);
       }
 
-#ifdef WINDOWS
-      str = g_strdup_printf("%s%s", PIXMAPDIR, iconfile2);
-      icon = gdk_pixbuf_new_from_file(str, NULL);
-      g_free(str);
-#else
-      icon = gdk_pixbuf_new_from_file(iconfile2, NULL);
-#endif
+      icon = gdk_pixbuf_new_from_resource(iconfile2, NULL);
       if (icon) {
 	list = g_list_append(list, icon);
       }
@@ -1438,7 +1456,7 @@ sub_window_create(struct SubWin *d, const char *title, GtkWidget *swin, const ch
 
   gtk_container_add(GTK_CONTAINER(dlg), swin);
 
-#ifdef WINDOWS
+#if WINDOWS
   g_signal_connect(dlg, "realize", G_CALLBACK(hide_minimize_menu_item), NULL);
 #endif
   g_signal_connect(dlg, "show", G_CALLBACK(cb_show), d);
@@ -1477,12 +1495,8 @@ label_sub_window_create(struct SubWin *d, const char *title, const char *icon, c
   GtkWidget *label, *swin;
 
   label = gtk_label_new(NULL);
-#if GTK_CHECK_VERSION(3, 4, 0)
   gtk_widget_set_halign(label, GTK_ALIGN_START);
   gtk_widget_set_valign(label, GTK_ALIGN_START);
-#else
-  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
-#endif
   gtk_label_set_selectable(GTK_LABEL(label), TRUE);
   gtk_label_set_line_wrap(GTK_LABEL(label), FALSE);
   gtk_label_set_single_line_mode(GTK_LABEL(label), FALSE);
@@ -1517,6 +1531,7 @@ list_widget_create(struct SubWin *d, int lisu_num, n_list_store *list, int can_f
   data = g_malloc0(sizeof(*data));
   data->select = -1;
   data->parent = d;
+  data->undo_save = NULL;
   data->can_focus = can_focus;
   data->list = list;
   data->list_col_num = lisu_num;
