@@ -12,21 +12,6 @@
 #include "x11gui.h"
 #include "dir_defs.h"
 
-GtkWidget *
-create_image_from_file(const char *file)
-{
-  GtkWidget *img;
-#ifdef WINDOWS
-    char *str;
-    str = g_strdup_printf("%s%s", PIXMAPDIR, file);
-    img = gtk_image_new_from_file(str);
-    g_free(str);
-#else
-    img = gtk_image_new_from_file(file);
-#endif
-    return img;
-}
-
 void
 set_button_icon(GtkWidget *w, const char *icon_name)
 {
@@ -40,7 +25,6 @@ set_button_icon(GtkWidget *w, const char *icon_name)
 void
 set_widget_margin(GtkWidget *w, int margin_pos)
 {
-#if GTK_CHECK_VERSION(3, 0, 0)
 #if GTK_CHECK_VERSION(3, 12, 0)
   if (margin_pos & WIDGET_MARGIN_LEFT) {
     gtk_widget_set_margin_start(w, 4);
@@ -66,7 +50,6 @@ set_widget_margin(GtkWidget *w, int margin_pos)
   if (margin_pos & WIDGET_MARGIN_TOP) {
     gtk_widget_set_margin_top(w, 4);
   }
-#endif
 }
 
 void
@@ -174,35 +157,19 @@ GtkWidget *
 add_widget_to_table_sub(GtkWidget *table, GtkWidget *w, char *title, int expand, int col, int width, int col_max, int n)
 {
   GtkWidget *label;
-#if ! GTK_CHECK_VERSION(3, 4, 0)
-  GtkWidget *align;
-  int x, y;
-
-  g_object_get(table, "n-columns", &x, "n-rows", &y, NULL);
-
-  x = (x > col_max) ? x : col_max;
-  y = (y > n + 1) ? y : n + 1;
-  gtk_table_resize(GTK_TABLE(table), y, x);
-#endif
 
   label = NULL;
 
   if (title) {
     label = gtk_label_new_with_mnemonic(title);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), w);
-#if GTK_CHECK_VERSION(3, 4, 0)
     gtk_widget_set_halign(label, GTK_ALIGN_START);
     g_object_set(label, "margin", GINT_TO_POINTER(4), NULL);
     gtk_grid_attach(GTK_GRID(table), label, col, n, 1, 1);
-#else
-    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, col, col + 1, n, n + 1, GTK_FILL, 0, 4, 4);
-#endif
     col++;
   }
 
   if (w) {
-#if GTK_CHECK_VERSION(3, 4, 0)
     if (expand) {
       gtk_widget_set_hexpand(w, TRUE);
       gtk_widget_set_halign(w, GTK_ALIGN_FILL);
@@ -211,11 +178,6 @@ add_widget_to_table_sub(GtkWidget *table, GtkWidget *w, char *title, int expand,
     }
     g_object_set(w, "margin", GINT_TO_POINTER(4), NULL);
     gtk_grid_attach(GTK_GRID(table), w, col, n, width, 1);
-#else
-    align = gtk_alignment_new(0, 0.5, (expand) ? 1 : 0, 0);
-    gtk_container_add(GTK_CONTAINER(align), w);
-    gtk_table_attach(GTK_TABLE(table), align, col, col + width, n, n + 1, ((expand) ? GTK_EXPAND : 0) | GTK_FILL, 0, 4, 4);
-#endif
   }
 
   return label;
@@ -232,11 +194,7 @@ add_copy_button_to_box(GtkWidget *parent_box, GCallback cb, gpointer d, char *ob
 {
   GtkWidget *hbox, *w;
 
-#if GTK_CHECK_VERSION(3, 0, 0)
   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
-#else
-  hbox = gtk_hbox_new(FALSE, 4);
-#endif
   w = gtk_button_new_with_mnemonic(_("_Copy Settings"));
   g_signal_connect(w, "map", G_CALLBACK(set_sensitivity_by_check_instance), obj_name);
   g_signal_connect(w, "clicked", cb, d);
@@ -251,7 +209,6 @@ item_setup(GtkWidget *box, GtkWidget *w, char *title, gboolean expand)
 {
   GtkWidget *hbox, *label;
 
-#if GTK_CHECK_VERSION(3, 0, 0)
   hbox = gtk_grid_new();
   label = gtk_label_new_with_mnemonic(title);
   gtk_label_set_mnemonic_widget(GTK_LABEL(label), w);
@@ -265,13 +222,6 @@ item_setup(GtkWidget *box, GtkWidget *w, char *title, gboolean expand)
   }
   gtk_grid_attach(GTK_GRID(hbox), label, 0, 0, 1, 1);
   gtk_grid_attach(GTK_GRID(hbox), w, 1, 0, 1, 1);
-#else
-  hbox = gtk_hbox_new(FALSE, 4);
-  label = gtk_label_new_with_mnemonic(title);
-  gtk_label_set_mnemonic_widget(GTK_LABEL(label), w);
-  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
-  gtk_box_pack_start(GTK_BOX(hbox), w, expand, expand, 2);
-#endif
   gtk_box_pack_start(GTK_BOX(box), hbox, expand, expand, 4);
 
   return label;
@@ -630,7 +580,7 @@ create_spin_entry_type(enum SPIN_BUTTON_TYPE type,
   case SPIN_BUTTON_TYPE_POINT:
     min = int2val(type, TEXT_SIZE_MIN);
     max = int2val(type, SPIN_ENTRY_MAX);
-    inc = 2;
+    inc = 1;
     break;
   case SPIN_BUTTON_TYPE_PERCENT:
     min = 0;
@@ -687,7 +637,27 @@ create_spin_entry(int min, int max, int inc,
 			    TRUE, FALSE, set_default_size, set_default_action);
 }
 
-#if GTK_CHECK_VERSION(3, 4, 0)
+static void
+set_custom_palette(GtkWidget *dlg)
+{
+  int n;
+  struct narray *palette;
+  GdkRGBA *colors;
+
+  palette = &(Menulocal.custom_palette);
+  n = arraynum(palette);
+  if (n < 1) {
+    return;
+  }
+  colors = arraydata(palette);
+  if (n >= 18) {
+    gtk_color_chooser_add_palette(GTK_COLOR_CHOOSER(dlg), GTK_ORIENTATION_HORIZONTAL, 9, n - 9, colors);
+    gtk_color_chooser_add_palette(GTK_COLOR_CHOOSER(dlg), GTK_ORIENTATION_HORIZONTAL, 9, 9, colors + (n - 9));
+  } else {
+    gtk_color_chooser_add_palette(GTK_COLOR_CHOOSER(dlg), GTK_ORIENTATION_HORIZONTAL, 9, n - 9, colors);
+  }
+}
+
 static void
 show_color_sel(GtkWidget *w, gpointer user_data)
 {
@@ -702,68 +672,84 @@ show_color_sel(GtkWidget *w, gpointer user_data)
 	   nround(col.blue * 255));
   gtk_widget_set_tooltip_text(w, buf);
 }
-#else
-static gboolean
-show_color_sel(GtkWidget *w, GdkEventButton *e, gpointer user_data)
+
+void
+add_default_color(struct narray *palette)
 {
-  GtkWidget *dlg;
-  GtkColorSelection *sel;
-  GdkColor col;
-  GtkColorButton *button;
-  gboolean r;
-  guint16 alpha;
-
-
-  button = GTK_COLOR_BUTTON(w);
-
-  gtk_color_button_get_color(button, &col);
-  alpha = (Menulocal.use_opacity) ? gtk_color_button_get_alpha(button) : 0xffff;
-
-  dlg = gtk_color_selection_dialog_new(_("Pick a Color"));
-  gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(user_data));
-  sel = GTK_COLOR_SELECTION(gtk_color_selection_dialog_get_color_selection(GTK_COLOR_SELECTION_DIALOG(dlg)));
-
-  gtk_color_selection_set_has_palette(sel, TRUE);
-  gtk_color_selection_set_has_opacity_control(sel, Menulocal.use_opacity);
-  gtk_color_selection_set_current_color(sel, &col);
-  gtk_color_selection_set_current_alpha(sel, alpha);
-
-  r = ndialog_run(dlg);
-  gtk_color_selection_get_current_color(sel, &col);
-  alpha = gtk_color_selection_get_current_alpha(sel);
-  gtk_widget_destroy(dlg);
-
-  if (r == GTK_RESPONSE_OK) {
-    char buf[64];
-    snprintf(buf, sizeof(buf),
-	     "#%02X%02X%02X",
-	     col.red >> 8,
-	     col.green >> 8,
-	     col.blue >> 8);
-    gtk_widget_set_tooltip_text(w, buf);
-
-    gtk_color_button_set_color(button, &col);
-    gtk_color_button_set_alpha(button, alpha);
+  const gchar *default_colors[9][3] = {
+    { "#ef2929", "#cc0000", "#a40000" }, /* Scarlet Red */
+    { "#fcaf3e", "#f57900", "#ce5c00" }, /* Orange */
+    { "#fce94f", "#edd400", "#c4a000" }, /* Butter */
+    { "#8ae234", "#73d216", "#4e9a06" }, /* Chameleon */
+    { "#729fcf", "#3465a4", "#204a87" }, /* Sky Blue */
+    { "#ad7fa8", "#75507b", "#5c3566" }, /* Plum */
+    { "#e9b96e", "#c17d11", "#8f5902" }, /* Chocolate */
+    { "#888a85", "#555753", "#2e3436" }, /* Aluminum 1 */
+    { "#eeeeec", "#d3d7cf", "#babdb6" }  /* Aluminum 2 */
+  };
+  GdkRGBA color;
+  int i, j;
+  for (j = 0; j < 3; j++) {
+    for (i = 0; i < 9; i++) {
+      gdk_rgba_parse(&color, default_colors[i][j]);
+      arrayadd(palette, &color);
+    }
   }
-
-  return TRUE;
 }
 
-static gboolean
-color_button_key_event(GtkWidget *w, GdkEventKey *e, gpointer u)
+void
+add_default_gray(struct narray *palette)
 {
-  switch (e->keyval) {
-  case GDK_KEY_space:
-  case GDK_KEY_Return:
-    if (e->type == GDK_KEY_RELEASE) {
-      show_color_sel(w, NULL, u);
-    }
-    return TRUE;
+  const gchar *default_grays[9] = {
+    "#000000", /* black */
+    "#2e3436", /* very dark gray */
+    "#555753", /* darker gray */
+    "#888a85", /* dark gray */
+    "#babdb6", /* medium gray */
+    "#d3d7cf", /* light gray */
+    "#eeeeec", /* lighter gray */
+    "#f3f3f3", /* very light gray */
+    "#ffffff"  /* white */
+  };
+  GdkRGBA color;
+  int i;
+  for (i = 0; i < 9; i++) {
+    gdk_rgba_parse(&color, default_grays[i]);
+    arrayadd(palette, &color);
   }
+}
 
+static void
+set_default_palette(GtkWidget *cc)
+{
+  struct narray palette;
+  gint n;
+
+  arrayinit(&palette, sizeof(GdkRGBA));
+  add_default_color(&palette);
+  n = arraynum(&palette);
+  gtk_color_chooser_add_palette(GTK_COLOR_CHOOSER(cc), GTK_ORIENTATION_HORIZONTAL, 9, n, arraydata(&palette));
+
+  arrayclear(&palette);
+  add_default_gray(&palette);
+  n = arraynum(&palette);
+  gtk_color_chooser_add_palette(GTK_COLOR_CHOOSER(cc), GTK_ORIENTATION_HORIZONTAL, n, n, arraydata(&palette));
+  arraydel(&palette);
+}
+
+#define CUSTOM_PALETTE_KEY "custom_palette"
+
+static gboolean
+show_color_dialog(GtkButton *btn, GdkEvent *event, gpointer user_data)
+{
+  gtk_color_chooser_add_palette(GTK_COLOR_CHOOSER(btn), GTK_ORIENTATION_HORIZONTAL, 0, 0, NULL);
+  if (Menulocal.use_custom_palette) {
+    set_custom_palette(GTK_WIDGET(btn));
+  } else {
+    set_default_palette(GTK_WIDGET(btn));
+  }
   return FALSE;
 }
-#endif
 
 GtkWidget *
 create_color_button(GtkWidget *win)
@@ -771,136 +757,13 @@ create_color_button(GtkWidget *win)
   GtkWidget *w;
 
   w = gtk_color_button_new();
-#if GTK_CHECK_VERSION(3, 4, 0)
+  g_object_set_data(G_OBJECT(w), CUSTOM_PALETTE_KEY, GINT_TO_POINTER(0));
   g_signal_connect(w, "color-set", G_CALLBACK(show_color_sel), win);
-#else
-  gtk_color_button_set_use_alpha(GTK_COLOR_BUTTON(w), Menulocal.use_opacity);
-  g_signal_connect(w, "button-release-event", G_CALLBACK(show_color_sel), win);
-  g_signal_connect(w, "key-press-event", G_CALLBACK(color_button_key_event), win);
-  g_signal_connect(w, "key-release-event", G_CALLBACK(color_button_key_event), win);
-#endif
+  g_signal_connect(w, "button-press-event", G_CALLBACK(show_color_dialog), win);
 
   return w;
 }
 
-static void
-set_adjustment(GtkAdjustment *adj, gdouble inc)
-{
-  gdouble val, min, max;
-
-  if (adj == NULL || inc == 0) {
-    return;
-  }
-
-  min = gtk_adjustment_get_lower(adj);
-  max = gtk_adjustment_get_upper(adj) - gtk_adjustment_get_page_size(adj);
-  val = gtk_adjustment_get_value(adj);
-  val += inc;
-
-  if (max < min) {
-    return;
-  }
-
-  if (val < min) {
-    val = min;
-  } else if (val > max) {
-    val = max;
-  }
-
-  gtk_adjustment_set_value(adj, val);
-}
-
-static gboolean
-text_view_scroll_event(GtkWidget *widget, GdkEventScroll *event, gpointer user_data)
-{
-  GtkRange *scl;
-  GtkAdjustment *x_adj, *y_adj;
-  gdouble x, y;
-
-  switch (event->direction) {
-  case GDK_SCROLL_UP:
-    scl = g_object_get_data(G_OBJECT(widget), "vscroll");
-    y_adj = gtk_range_get_adjustment(scl);
-    y = - gtk_adjustment_get_step_increment(y_adj);
-    x_adj = NULL;
-    x = 0;
-    break;
-  case GDK_SCROLL_DOWN:
-    scl = g_object_get_data(G_OBJECT(widget), "vscroll");
-    y_adj = gtk_range_get_adjustment(scl);
-    y = gtk_adjustment_get_step_increment(y_adj);
-    x_adj = NULL;
-    x = 0;
-    break;
-  case GDK_SCROLL_LEFT:
-    scl = g_object_get_data(G_OBJECT(widget), "hscroll");
-    x_adj = gtk_range_get_adjustment(scl);
-    x = - gtk_adjustment_get_step_increment(x_adj);
-    y_adj = NULL;
-    y = 0;
-    break;
-  case GDK_SCROLL_RIGHT:
-    scl = g_object_get_data(G_OBJECT(widget), "hscroll");
-    x_adj = gtk_range_get_adjustment(scl);
-    x = gtk_adjustment_get_step_increment(x_adj);
-    y_adj = NULL;
-    y = 0;
-    break;
-#if GTK_CHECK_VERSION(3, 4, 0)
-  case GDK_SCROLL_SMOOTH:
-    if (gdk_event_get_scroll_deltas((GdkEvent *) event, &x, &y)) {
-      scl = g_object_get_data(G_OBJECT(widget), "hscroll");
-      x_adj = gtk_range_get_adjustment(scl);
-      x *= gtk_adjustment_get_step_increment(x_adj);
-
-      scl = g_object_get_data(G_OBJECT(widget), "vscroll");
-      y_adj = gtk_range_get_adjustment(scl);
-      y *= gtk_adjustment_get_step_increment(y_adj);
-      break;
-    }
-    return FALSE;
-#endif
-  default:
-    return FALSE;
-  }
-
-  set_adjustment(x_adj, x);
-  set_adjustment(y_adj, y);
-
-  return FALSE;
-}
-
-
-static void
-set_scroll_visibility(GtkWidget *scroll)
-{
-  GtkAdjustment *adj;
-  gdouble min, max, page;
-
-  if (scroll == NULL) {
-    return;
-  }
-
-  adj = gtk_range_get_adjustment(GTK_RANGE(scroll));
-  min = gtk_adjustment_get_lower(adj);
-  max = gtk_adjustment_get_upper(adj);
-  page = gtk_adjustment_get_page_size(adj);
-
-  gtk_widget_set_visible(scroll, max - min > page);
-}
-
-static void
-text_view_size_allocate(GtkWidget*widget, GdkRectangle *allocation, gpointer user_data)
-{
-  GtkWidget *scl;
-
-  scl = g_object_get_data(G_OBJECT(widget), "hscroll");
-  set_scroll_visibility(scl);
-
-  scl = g_object_get_data(G_OBJECT(widget), "vscroll");
-  set_scroll_visibility(scl);
-}
-
 #if GTK_CHECK_VERSION(3, 16, 0)
 void
 set_widget_font(GtkWidget *w, const char *font)
@@ -986,7 +849,12 @@ set_widget_font(GtkWidget *w, const char *font)
     unit = "pt";
   }
 
-  css_str = g_strdup_printf("* {font: %s %d %d%s \"%s\";}",
+  css_str = g_strdup_printf("* {\n"
+			    "   font-style: %s;\n"
+			    "   font-weight: %d;\n"
+			    "   font-size: %d%s;\n"
+			    "   font-family: \"%s\";\n"
+			    "}",
 			    style_str,
 			    weight_val,
 			    size / PANGO_SCALE,
@@ -1021,7 +889,6 @@ set_widget_font(GtkWidget *w, const char *font)
 static void
 set_linumber_color(GtkWidget *w)
 {
-#if GTK_CHECK_VERSION(3, 0, 0)
   GdkRGBA col;
 
   col.red   = LINE_NUMBER_R * 1.0 / 0xFFFF;
@@ -1045,191 +912,43 @@ set_linumber_color(GtkWidget *w)
   gtk_widget_override_color(w, GTK_STATE_FLAG_PRELIGHT, &col);
   gtk_widget_override_color(w, GTK_STATE_FLAG_SELECTED, &col);
   gtk_widget_override_color(w, GTK_STATE_FLAG_INSENSITIVE, &col);
-#else
-  GdkColor col;
-
-  col.red = LINE_NUMBER_R;
-  col.green = LINE_NUMBER_G;
-  col.blue = LINE_NUMBER_B;
-
-  gtk_widget_modify_base(w, GTK_STATE_NORMAL, &col);
-  gtk_widget_modify_base(w, GTK_STATE_ACTIVE, &col);
-  gtk_widget_modify_base(w, GTK_STATE_PRELIGHT, &col);
-  gtk_widget_modify_base(w, GTK_STATE_SELECTED, &col);
-  gtk_widget_modify_base(w, GTK_STATE_INSENSITIVE, &col);
-
-  col.red = 0;
-  col.green = 0;
-  col.blue = 0;
-
-  gtk_widget_modify_text(w, GTK_STATE_NORMAL, &col);
-  gtk_widget_modify_text(w, GTK_STATE_ACTIVE, &col);
-  gtk_widget_modify_text(w, GTK_STATE_PRELIGHT, &col);
-  gtk_widget_modify_text(w, GTK_STATE_SELECTED, &col);
-  gtk_widget_modify_text(w, GTK_STATE_INSENSITIVE, &col);
-#endif
 
   gtk_widget_set_sensitive(w, FALSE);
 }
 
 #endif	/* ! GTK_CHECK_VERSION(3, 16, 0) */
 
-#if GTK_CHECK_VERSION(3, 0, 0)
-static void (* get_preferred_width_org) (GtkWidget *w, gint *min, gint *natulal);
-static void (* get_preferred_height_org) (GtkWidget *w, gint *min, gint *natulal);
-
-static void
-get_preferred_width(GtkWidget *w, gint *min, gint *natulal)
-{
-  get_preferred_width_org(w, min, natulal);
-  if (*min > 240) {
-    *min = 0;
-  }
-}
-
-static void
-get_preferred_height(GtkWidget *w, gint *min, gint *natulal)
-{
-  get_preferred_height_org(w, min, natulal);
-  *min = 0;
-}
-#endif
-
 GtkWidget *
 create_text_view_with_line_number(GtkWidget **v)
 {
-  GtkWidget *view, *ln, *swin, *hs, *vs;
-  GtkAdjustment *hadj, *vadj;
-  GtkTextBuffer *buf;
+  GtkWidget *source_view, *swin;
+  GtkSourceBuffer *buffer;
 
-  buf = gtk_text_buffer_new(NULL);
-  view = gtk_text_view_new_with_buffer(buf);
+  source_view = gtk_source_view_new();
+  gtk_widget_set_hexpand(source_view, TRUE);
+  gtk_widget_set_vexpand(source_view, TRUE);
+  buffer = gtk_source_buffer_new(NULL);
 
-  buf = gtk_text_buffer_new(NULL);
-  ln = gtk_text_view_new_with_buffer(buf);
-#if GTK_CHECK_VERSION(3, 12, 0)
-  gtk_widget_set_margin_end(ln, 4);
-#endif
+  gtk_text_view_set_buffer(GTK_TEXT_VIEW(source_view), GTK_TEXT_BUFFER(buffer));
+  gtk_text_view_set_editable(GTK_TEXT_VIEW(source_view), FALSE);
 
-#if GTK_CHECK_VERSION(3, 16, 0)
-  gtk_widget_set_name(ln, LINE_NUMBER_WIDGET_NAME);
-#else
-  set_linumber_color(ln);
-#endif
-
-  g_object_set_data(G_OBJECT(view), "line_number", ln);
-
-  gtk_widget_set_size_request(GTK_WIDGET(view), 240, 120);
-  gtk_widget_set_size_request(GTK_WIDGET(ln),   -1,  120);
-
-  gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
-  gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view), FALSE);
-
-  gtk_text_view_set_editable(GTK_TEXT_VIEW(ln), FALSE);
-  gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(ln), FALSE);
-
-#if GTK_CHECK_VERSION(3, 4, 0)
-  swin = gtk_grid_new();
-#else
-  swin = gtk_table_new(3, 2, FALSE);
-#endif
-#if GTK_CHECK_VERSION(3, 2, 0)
-  hs = gtk_scrollbar_new(GTK_ORIENTATION_HORIZONTAL, NULL);
-  vs = gtk_scrollbar_new(GTK_ORIENTATION_VERTICAL, NULL);
-#else
-  hs = gtk_hscrollbar_new(NULL);
-  vs = gtk_vscrollbar_new(NULL);
-#endif
-
-  g_object_set_data(G_OBJECT(view), "hscroll", hs);
-  g_object_set_data(G_OBJECT(view), "vscroll", vs);
-
-  hadj = gtk_range_get_adjustment(GTK_RANGE(hs));
-  vadj = gtk_range_get_adjustment(GTK_RANGE(vs));
-#if GTK_CHECK_VERSION(3, 0, 0)
-  gtk_scrollable_set_hadjustment(GTK_SCROLLABLE(view), hadj);
-  gtk_scrollable_set_vadjustment(GTK_SCROLLABLE(view), vadj);
-  gtk_scrollable_set_vadjustment(GTK_SCROLLABLE(ln), vadj);
-
-  /* fix-me: is there any other way to set minimum size of GtkTextView? */
-  get_preferred_width_org = GTK_WIDGET_GET_CLASS(view)->get_preferred_width;
-  get_preferred_height_org = GTK_WIDGET_GET_CLASS(view)->get_preferred_height;
-  GTK_WIDGET_GET_CLASS(view)->get_preferred_width = get_preferred_width;
-  GTK_WIDGET_GET_CLASS(view)->get_preferred_height = get_preferred_height;
-#else
-  gtk_widget_set_scroll_adjustments(view, hadj, vadj);
-  gtk_widget_set_scroll_adjustments(ln,   NULL, vadj);
-#endif
-
-  g_signal_connect(view, "scroll-event", G_CALLBACK(text_view_scroll_event), NULL);
-  g_signal_connect(view, "size-allocate", G_CALLBACK(text_view_size_allocate), NULL);
-
-#if GTK_CHECK_VERSION(3, 4, 0)
-  gtk_widget_set_vexpand(ln, TRUE);
-  gtk_grid_attach(GTK_GRID(swin), ln,   0, 0, 1, 1);
-
-  gtk_widget_set_hexpand(view, TRUE);
-  gtk_widget_set_vexpand(view, TRUE);
-  gtk_grid_attach(GTK_GRID(swin), view, 1, 0, 1, 1);
-
-  gtk_widget_set_vexpand(hs, TRUE);
-  gtk_grid_attach(GTK_GRID(swin), hs,   0, 1, 2, 1);
-
-  gtk_widget_set_vexpand(vs, TRUE);
-  gtk_grid_attach(GTK_GRID(swin), vs,   2, 0, 1, 1);
-#else
-  gtk_table_attach(GTK_TABLE(swin), ln,   0, 1, 0, 1,
-		   0, GTK_EXPAND | GTK_FILL, 0, 0);
-  gtk_table_attach(GTK_TABLE(swin), view, 1, 2, 0, 1,
-		   GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-  gtk_table_attach(GTK_TABLE(swin), hs,   0, 2, 1, 2,
-		   GTK_FILL, GTK_FILL, 0, 0);
-  gtk_table_attach(GTK_TABLE(swin), vs,   2, 3, 0, 1,
-		   GTK_FILL, GTK_FILL, 0, 0);
-#endif
-
-  if (v) {
-    *v = view;
-  }
+  gtk_source_buffer_set_highlight_syntax(GTK_SOURCE_BUFFER(buffer), FALSE);
+  gtk_source_view_set_show_line_numbers(GTK_SOURCE_VIEW(source_view), TRUE);
 
+  swin = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  gtk_container_add(GTK_CONTAINER(swin), source_view);
+  *v = source_view;
   return swin;
 }
 
 void
 text_view_with_line_number_set_text(GtkWidget *view, const gchar *str)
 {
-  GtkWidget *ln;
-  GtkTextBuffer *buf, *ln_buf;
-  int p, i, n;
-  GString *s;
-  gchar *ptr;
-  GtkTextIter start, end;
+  GtkTextBuffer *buf;
 
   buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
   gtk_text_buffer_set_text(buf, str, -1);
-
-  ln = g_object_get_data(G_OBJECT(view), "line_number");
-  if (ln == NULL) {
-    return;
-  }
-
-  s = g_string_sized_new(256);
-  if (s == NULL) {
-    return;
-  }
-
-  n = gtk_text_buffer_get_line_count(buf);
-  ln_buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ln));
-  p = ceil(log10(n + 1));
-  for (i = 0; i < n; i++) {
-    g_string_append_printf(s, "%*d \n", p, i + 1);
-  }
-  ptr = g_string_free(s, FALSE);
-  gtk_text_buffer_set_text(ln_buf, ptr, -1);
-  g_free(ptr);
-
-  gtk_text_buffer_get_iter_at_offset(ln_buf, &start, 0);
-  gtk_text_buffer_get_iter_at_offset(ln_buf, &end, -1);
 }
 
 void
@@ -1238,41 +957,13 @@ text_view_with_line_number_set_font(GtkWidget *view, const gchar *font)
 #if ! GTK_CHECK_VERSION(3, 16, 0)
   PangoFontDescription *desc;
 #endif
-  GtkWidget *ln;
 
 #if GTK_CHECK_VERSION(3, 16, 0)
   set_widget_font(view, font);
 #else  /* GTK_CHECK_VERSION(3, 16, 0) */
   desc = pango_font_description_from_string(font);
-#if GTK_CHECK_VERSION(3, 0, 0)
   gtk_widget_override_font(view, NULL);
   gtk_widget_override_font(view, desc);
-#else
-  gtk_widget_modify_font(view, NULL);
-  gtk_widget_modify_font(view, desc);
-#endif
-#endif	/* GTK_CHECK_VERSION(3, 16, 0) */
-
-  ln = g_object_get_data(G_OBJECT(view), "line_number");
-  if (ln == NULL) {
-#if ! GTK_CHECK_VERSION(3, 16, 0)
-    pango_font_description_free(desc);
-#endif
-    return;
-  }
-
-#if GTK_CHECK_VERSION(3, 16, 0)
-  set_widget_font(ln, font);
-#else  /* GTK_CHECK_VERSION(3, 16, 0) */
-#if GTK_CHECK_VERSION(3, 0, 0)
-  gtk_widget_override_font(ln, NULL);
-  gtk_widget_override_font(ln, desc);
-#else
-  gtk_widget_modify_font(ln, NULL);
-  gtk_widget_modify_font(ln, desc);
-#endif
-
-  pango_font_description_free(desc);
 #endif	/* GTK_CHECK_VERSION(3, 16, 0) */
 }
 
@@ -1280,13 +971,8 @@ enum SELECT_OBJ_COLOR_RESULT
 select_obj_color(struct objlist *obj, int id, enum OBJ_FIELD_COLOR_TYPE type)
 {
   GtkWidget *dlg;
-  int r, g, b, a, rr ,gg, bb, aa, response;
-#if GTK_CHECK_VERSION(3, 4, 0)
+  int r, g, b, a, rr ,gg, bb, aa, response, modified, undo;
   GdkRGBA color;
-#else
-  GtkWidget *sel;
-  GdkColor color;
-#endif
   char *title;
 
   switch (type) {
@@ -1354,16 +1040,17 @@ select_obj_color(struct objlist *obj, int id, enum OBJ_FIELD_COLOR_TYPE type)
     a = 255;
   }
 
-#if GTK_CHECK_VERSION(3, 4, 0)
   color.red = r / 255.0;
   color.green = g / 255.0;
   color.blue = b / 255.0;
   color.alpha = a / 255.0;
 
   dlg = gtk_color_chooser_dialog_new(title, GTK_WINDOW(NgraphApp.FileWin.Win));
+  if (Menulocal.use_custom_palette) {
+    set_custom_palette(dlg);
+  }
   gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(dlg), &color);
   gtk_color_chooser_set_use_alpha(GTK_COLOR_CHOOSER(dlg), Menulocal.use_opacity);
-
   response = ndialog_run(dlg);
   gtk_color_chooser_get_rgba(GTK_COLOR_CHOOSER(dlg), &color);
 
@@ -1377,35 +1064,8 @@ select_obj_color(struct objlist *obj, int id, enum OBJ_FIELD_COLOR_TYPE type)
   gg = nround(color.green * 255);
   bb = nround(color.blue * 255);
   aa = nround(color.alpha * 255);
-#else
-  color.red = (r & 0xffU) * 257;
-  color.green = (g & 0xffU) * 257;
-  color.blue = (b & 0xffU) * 257;
-
-  dlg = gtk_color_selection_dialog_new(title);
-  sel = gtk_color_selection_dialog_get_color_selection(GTK_COLOR_SELECTION_DIALOG(dlg));
-
-  gtk_color_selection_set_has_palette(GTK_COLOR_SELECTION(sel), TRUE);
-  gtk_color_selection_set_has_opacity_control(GTK_COLOR_SELECTION(sel), Menulocal.use_opacity);
-
-  gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(sel), &color);
-  gtk_color_selection_set_current_alpha(GTK_COLOR_SELECTION(sel), (a & 0xffU) * 257);
-
-  response = ndialog_run(dlg);
-  gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(sel), &color);
-  aa = gtk_color_selection_get_current_alpha(GTK_COLOR_SELECTION(sel));
-  gtk_widget_destroy(dlg);
-
-  if (response != GTK_RESPONSE_OK) {
-    return SELECT_OBJ_COLOR_CANCEL;
-  }
-
-  rr = (color.red >> 8);
-  gg = (color.green >> 8);
-  bb = (color.blue >> 8);
-  aa >>= 8;
-#endif
 
+  undo = menu_save_undo_single(UNDO_TYPE_EDIT, obj->name);
   switch (type) {
   case OBJ_FIELD_COLOR_TYPE_STROKE:
     putobj(obj, "stroke_R", id, &rr);
@@ -1449,5 +1109,11 @@ select_obj_color(struct objlist *obj, int id, enum OBJ_FIELD_COLOR_TYPE type)
     return SELECT_OBJ_COLOR_ERROR;
   }
 
-  return (rr == r && gg == g && bb == b && aa == a) ? SELECT_OBJ_COLOR_SAME : SELECT_OBJ_COLOR_DIFFERENT;
+  if (rr == r && gg == g && bb == b && aa == a) {
+    modified = SELECT_OBJ_COLOR_SAME;
+    menu_delete_undo(undo);
+  } else {
+    modified = SELECT_OBJ_COLOR_DIFFERENT;
+  }
+  return modified;
 }
