@@ -1,24 +1,24 @@
-/* 
+/*
  * $Id: omerge.c,v 1.16 2010-03-04 08:30:16 hito Exp $
- * 
+ *
  * This file is part of "Ngraph for X11".
- * 
+ *
  * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
- * 
+ *
  * "Ngraph for X11" is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * "Ngraph for X11" is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- * 
+ *
  */
 
 #include "common.h"
@@ -40,11 +40,14 @@
 #include "gra.h"
 #include "oroot.h"
 #include "odraw.h"
+#include "osystem.h"
 
 #define NAME "merge"
 #define PARENT "draw"
 #define OVERSION  "1.00.00"
 
+#define OLD_GRA_HEADER " Ngraph GRA file"
+
 #define ERRFILE 100
 #define ERROPEN 101
 #define ERRGRA  102
@@ -62,9 +65,131 @@ static char *mergeerrorlist[]={
 struct mergelocal {
   FILE *storefd;
   int endstore;
+  time_t mtime;
+  int bbox[4];
 };
 
-static int 
+struct gra_info
+{
+  int GC, lm, tm, zm;
+  GStatBuf gstat;
+  struct objlist *obj;
+};
+
+#define USE_MERGE_CACHE 1
+
+#if USE_MERGE_CACHE
+struct gra_cache
+{
+  struct GRAdata *data;
+  GStatBuf gstat;
+};
+
+static NHASH GraCache = NULL;
+#endif
+
+static void
+set_bbox(struct objlist *obj,N_VALUE *inst, struct narray *array, int l, int t, int z)
+{
+  int *bbox;
+  double zoom;
+  struct mergelocal *mergelocal;
+  _getobj(obj,"_local",inst,&mergelocal);
+  if (arraynum(array) != 4) {
+    return;
+  }
+  bbox = arraydata(array);
+  zoom = z / 10000.0;
+  bbox[0] = (mergelocal->bbox[0]) * zoom + l;
+  bbox[1] = (mergelocal->bbox[1]) * zoom + t;
+  bbox[2] = (mergelocal->bbox[2]) * zoom + l;
+  bbox[3] = (mergelocal->bbox[3]) * zoom + t;
+}
+
+#if USE_MERGE_CACHE
+static struct GRAdata *
+gra_data_new(void)
+{
+  struct GRAdata *data;
+  data = g_malloc(sizeof(*data));
+
+  if (data == NULL) {
+    return NULL;
+  }
+  data->code = '\0';
+  data->cpar = NULL;
+  data->cstr = NULL;
+  data->next = NULL;
+  return data;
+}
+
+static void
+gra_data_free(struct GRAdata *data)
+{
+  struct GRAdata *next;
+  while (data) {
+    next = data->next;
+    g_free(data->cpar);
+    g_free(data->cstr);
+    g_free(data);
+    data = next;
+  }
+}
+
+static struct gra_cache *
+gra_cache_new(const char *file)
+{
+  struct gra_cache *cache;
+
+  if (GraCache == NULL) {
+    GraCache = nhash_new();
+  }
+
+  if (GraCache == NULL) {
+    return NULL;
+  }
+
+  cache = g_malloc(sizeof(*cache));
+  if (cache == NULL) {
+    return NULL;
+  }
+  cache->data = NULL;
+  cache->gstat.st_mtime = 0;
+  cache->gstat.st_size = 0;
+
+  nhash_set_ptr(GraCache, file, cache);
+  return cache;
+}
+
+static struct gra_cache *
+gra_cache_get(const char *file)
+{
+  void *ptr;
+  if (GraCache == NULL) {
+    return NULL;
+  }
+
+  nhash_get_ptr(GraCache, file, &ptr);
+  return ptr;
+}
+  
+static void
+gra_cache_free(const char *file)
+{
+  struct gra_cache *cache;
+
+  cache = gra_cache_get(file);
+  if (cache == NULL) {
+    return;
+  }
+
+  gra_data_free(cache->data);
+  g_free(cache);
+  nhash_del(GraCache, file);
+}
+#endif  /* USE_MERGE_CACHE */
+
+static int
 mergeinit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   int zm, n;
@@ -85,6 +210,11 @@ mergeinit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 
   mergelocal->storefd=NULL;
   mergelocal->endstore=FALSE;
+  mergelocal->mtime = 0;
+  mergelocal->bbox[0] = 0;
+  mergelocal->bbox[1] = 0;
+  mergelocal->bbox[2] = 0;
+  mergelocal->bbox[3] = 0;
   return 0;
 
 errexit:
@@ -92,38 +222,301 @@ errexit:
   return 1;
 }
 
-static int 
+static int
 mergedone(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   if (_exeparent(obj,(char *)argv[1],inst,rval,argc,argv)) return 1;
   return 0;
 }
 
-static int 
-mergedraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
+#if USE_MERGE_CACHE
+static void
+gra_cache_init(struct gra_cache *cache, struct gra_info *info)
+{
+  if (cache->data) {
+    gra_data_free(cache->data);
+    cache->data = NULL;
+  }
+  if (info) {
+    cache->gstat = info->gstat;
+  } else {
+    cache->gstat.st_mtime = 0;
+    cache->gstat.st_size = 0;
+  }
+}
+
+static int *
+dup_cpar(const int *cpar)
+{
+  int n;
+
+  n = cpar[0] + 1;
+  if (n < 1) {
+    n = 1;
+  }
+  return g_memdup(cpar, n * sizeof(*cpar));
+}
+
+static int
+draw_gra_data(struct gra_info *info, struct GRAdata *data)
+{
+  int rcode, *cpar;
+  cpar = dup_cpar(data->cpar);
+  if (cpar == NULL) {
+    return FALSE;
+  }
+  rcode = GRAinputdraw(info->GC, info->lm, info->tm, info->zm, data->code, cpar, data->cstr);
+  g_free(cpar);
+  return rcode;
+}
+
+static int
+read_new_gra_file(struct gra_cache *cache, struct gra_info *info, FILE *fd)
+{
+  int r;
+  char *buf;
+  int rcode;
+  struct GRAdata *prev, *data;
+
+  gra_cache_init(cache, info);
+
+  r = 0;
+  prev = NULL;
+  while ((rcode = fgetline(fd, &buf)) != 1) {
+    if (rcode == -1) {
+      return 1;
+    }
+    data = gra_data_new();
+    if (data == NULL) {
+      goto errexit;
+    }
+    rcode = GRAparse(data, buf);
+    if (! rcode) {
+      GRAdata_free(data);
+      error2(info->obj, ERRGRAFM, buf);
+      goto errexit;
+    }
+    rcode = draw_gra_data(info, data);
+    if (! rcode) {
+      GRAdata_free(data);
+      error2(info->obj, ERRGRAFM, buf);
+      goto errexit;
+    }
+    g_free(buf);
+    if (prev) {
+      prev->next = data;
+    } else {
+      cache->data = data;
+    }
+    prev = data;
+  }
+  return r;
+
+errexit:
+  g_free(buf);
+  gra_cache_init(cache, NULL);
+  return 1;
+}
+
+static int
+read_old_gra_file(struct gra_cache *cache, struct gra_info *info, FILE *fd, const char *file)
+{
+  int r;
+  char *buf;
+  int rcode;
+
+  gra_cache_init(cache, NULL);
+  rcode = fgetline(fd, &buf);
+  if (rcode == 1) {
+    error2(info->obj, ERRGRA, file);
+  }
+  g_free(buf);
+  if (rcode != 0) {
+    return 1;
+  }
+
+  r = 0;
+  while ((rcode = fgetline(fd, &buf)) != 1) {
+    if (rcode == -1) {
+      return 1;
+    }
+    rcode = GRAinputold(info->GC, buf, info->lm, info->tm, info->zm);
+    if (!rcode) {
+      error2(info->obj, ERRGRAFM, buf);
+      g_free(buf);
+      return 1;
+    }
+    g_free(buf);
+  }
+  return r;
+}
+
+static int
+read_gra_file(struct gra_cache *cache, struct gra_info *info, const char *graf, const char *file)
 {
-  int GC;
-  char *file,*graf;
-  struct objlist *sys;
   FILE *fd;
   char *buf;
-  int lm,tm,zm;
-  int newgra,rcode,line = 0;
+  int newgra, rcode;
+
+  fd = nfopen(file, "rt");
+  if (fd == NULL) {
+    error2(info->obj, ERROPEN, file);
+    return 1;
+  }
+
+  rcode = fgetline(fd, &buf);
+  if (rcode == 1) {
+    error2(info->obj,ERRGRA,file);
+  }
+  if (rcode != 0) {
+    error2(info->obj, ERROPEN, file);
+    fclose(fd);
+    return 1;
+  }
+  if (strcmp(graf, buf) == 0) {
+    newgra = TRUE;
+  } else if (strcmp(OLD_GRA_HEADER, buf) == 0) {
+    newgra = FALSE;
+  } else {
+    error2(info->obj, ERRGRA, file);
+    g_free(buf);
+    fclose(fd);
+    return 1;
+  }
+  g_free(buf);
+
+  if (newgra) {
+    rcode = read_new_gra_file(cache, info, fd);
+  } else {
+    rcode = read_old_gra_file(cache, info, fd, file);
+  }
+  fclose(fd);
+  return rcode;
+}
+
+static int
+draw_gra(struct gra_cache *cache, struct gra_info *info)
+{
+  struct GRAdata *data;
+  int rcode;
+
+  if (cache == NULL) {
+    return 0;
+  }
+
+  data = cache->data;
+  if (data == NULL) {
+    return 0;
+  }
+  while (data) {
+    rcode = draw_gra_data(info, data);
+    if (! rcode) {
+      return 1;
+    }
+    data = data->next;
+  }
+  return 0;
+}
+
+static int
+free_cache(struct nhash *hash, void *data)
+{
+  struct gra_cache *cache;
+
+  cache = hash->val.p;
+  if (cache == NULL) {
+    return 0;
+  }
+
+  gra_data_free(cache->data);
+  g_free(cache);
+  hash->val.p = NULL;
+
+  return 0;
+}
+
+static int
+read_gra(struct gra_info *info, const char *graf, const char *file)
+{
+  struct gra_cache *cache;
+
+  cache = gra_cache_get(file);
+  if (cache == NULL) {
+    cache = gra_cache_new(file);
+    if (cache == NULL) {
+      return 1;
+    }
+  }
+  if (info->gstat.st_mtime != cache->gstat.st_mtime ||
+      info->gstat.st_size != cache->gstat.st_size) {
+    if (read_gra_file(cache, info, graf, file)) {
+      gra_cache_free(file);
+      return 1;
+    }
+  } else {
+    if (draw_gra(cache, info)) {
+      error2(info->obj, ERRGRAFM, file);
+      return 1;
+    }
+  }
+
+  return 0;
+}
+#endif  /* USE_MERGE_CACHE */
+
+void
+merge_cache_clear(void)
+{
+#if USE_MERGE_CACHE
+  if (GraCache == NULL) {
+    return;
+  }
+  nhash_each(GraCache, free_cache, NULL);
+  nhash_clear(GraCache);
+#endif
+}
+
+static int
+mergedraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
+{
+  struct gra_info info;
+  char *file,*graf;
+  struct objlist *sys;
+  struct mergelocal *mergelocal;
+#if ! USE_MERGE_CACHE
+  FILE *fd;
+  int newgra,rcode;
+  char *buf;
+#endif
 
   if (_exeparent(obj,(char *)argv[1],inst,rval,argc,argv)) return 1;
-  _getobj(obj,"GC",inst,&GC);
-  if (GC<0) return 0;
+  _getobj(obj,"GC",inst,&info.GC);
+  if (info.GC<0) return 0;
   _getobj(obj,"file",inst,&file);
-  _getobj(obj,"left_margin",inst,&lm);
-  _getobj(obj,"top_margin",inst,&tm);
-  _getobj(obj,"zoom",inst,&zm);
-
+  _getobj(obj,"left_margin",inst,&info.lm);
+  _getobj(obj,"top_margin",inst,&info.tm);
+  _getobj(obj,"zoom",inst,&info.zm);
+  _getobj(obj,"_local",inst,&mergelocal);
+  info.obj = obj;
+  
   if (file==NULL) {
     error(obj,ERRFILE);
     return 1;
   }
+  if (nstat(file, &info.gstat)) {
+    clear_bbox(obj, inst);
+    error2(obj, ERROPEN, file);
+    return 1;
+  }
+  if (info.gstat.st_mtime != mergelocal->mtime) {
+    clear_bbox(obj, inst);
+  }
+  mergelocal->mtime = info.gstat.st_mtime;
   if ((sys=getobject("system"))==NULL) return 1;
   if (getobj(sys,"GRAF",0,0,NULL,&graf)) return 1;
+#if USE_MERGE_CACHE
+  read_gra(&info, graf, file);
+#else  /* USE_MERGE_CACHE */
   if ((fd=nfopen(file,"rt"))==NULL) {
     error2(obj,ERROPEN,file);
     return 1;
@@ -135,7 +528,7 @@ mergedraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
     return 1;
   }
   if (strcmp(graf,buf)==0) newgra=TRUE;
-  else if (strcmp(" Ngraph GRA file",buf)==0) newgra=FALSE;
+  else if (strcmp(OLD_GRA_HEADER,buf)==0) newgra=FALSE;
   else {
     error2(obj,ERRGRA,file);
     g_free(buf);
@@ -156,8 +549,11 @@ mergedraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
       fclose(fd);
       return 1;
     }
-    if (newgra) rcode=GRAinput(GC,buf,lm,tm,zm);
-    else rcode=GRAinputold(GC,buf,lm,tm,zm);
+    if (newgra) {
+      rcode=GRAinput(info.GC, buf, info.lm, info.tm, info.zm);
+    } else {
+      rcode=GRAinputold(info.GC, buf, info.lm, info.tm, info.zm);
+    }
     if (!rcode) {
       error2(obj,ERRGRAFM,buf);
       g_free(buf);
@@ -165,37 +561,64 @@ mergedraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
       return 1;
     }
     g_free(buf);
-    line++;
   }
   fclose(fd);
-  GRAaddlist(GC,obj,inst,(char *)argv[0],"redraw");
-  _putobj(obj, "line_num", inst, &line);
+#endif  /* USE_MERGE_CACHE */
+  GRAaddlist(info.GC,obj,inst,(char *)argv[0],"redraw");
   return 0;
 }
 
-static int 
+static int
 mergeredraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
-  int redrawf, dmax, line_num;
-  int GC;
+  int hidden;
+#if ! USE_MERGE_CACHE
+  int redrawf;
+#endif
 
   if (_exeparent(obj,(char *)argv[1],inst,rval,argc,argv)) return 1;
-  _getobj(obj,"redraw_flag",inst,&redrawf);
-  _getobj(obj,"redraw_num", inst, &dmax);
-  _getobj(obj,"line_num", inst, &line_num);
+  _getobj(obj,"hidden",inst,&hidden);
 
-  if (redrawf && line_num > 0 && (dmax == 0 || line_num < dmax * 10)) {
+#if USE_MERGE_CACHE
+  if (! hidden) {
+    mergedraw(obj,inst,rval,argc,argv);
+  }
+#else  /* USE_MERGE_CACHE */
+  _getobj(obj,"redraw_flag",inst,&redrawf);
+  if (redrawf) {
     mergedraw(obj,inst,rval,argc,argv);
   } else {
+    int GC;
+    struct narray *array;
+    int x1, x2, y1, y2, zoom, w, h;
+    if (! hidden) {
+      system_draw_notify();
+    }
     _getobj(obj,"GC",inst,&GC);
     if (GC<0) return 0;
+    if (_exeobj(obj, "bbox", inst, 0, NULL)) {
+      return 1;
+    }
+    _getobj(obj, "bbox", inst, &array);
+    if (array) {
+      x1 = arraynget_int(array,0);
+      y1 = arraynget_int(array,1);
+      x2 = arraynget_int(array,2);
+      y2 = arraynget_int(array,3);
+      GRAregion(GC, &w, &h, &zoom);
+      GRAview(GC, 0, 0, w * 10000.0 / zoom, h * 10000.0 / zoom, 1);
+      GRAcolor(GC, 0, 0, 0, 255);
+      GRAlinestyle(GC, 0, NULL, 10, GRA_LINE_CAP_BUTT, GRA_LINE_JOIN_MITER, 1000);
+      GRArectangle(GC, x1, y1, x2, y2, 0);
+    }
     GRAaddlist(GC,obj,inst,(char *)argv[0],(char *)argv[1]);
   }
+#endif  /* USE_MERGE_CACHE */
   return 0;
 }
 
-static int 
-mergefile(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, 
+static int
+mergefile(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
             int argc, char **argv)
 {
   struct objlist *sys;
@@ -231,7 +654,7 @@ mergefile(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
   return 0;
 }
 
-static int 
+static int
 mergetime(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   char *file;
@@ -248,7 +671,7 @@ mergetime(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   return 0;
 }
 
-static int 
+static int
 mergedate(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   char *file;
@@ -265,7 +688,7 @@ mergedate(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   return 0;
 }
 
-static int 
+static int
 mergestore(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   struct mergelocal *mergelocal;
@@ -329,7 +752,7 @@ mergestore(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   }
 }
 
-static int 
+static int
 mergeload(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   char *s;
@@ -380,7 +803,7 @@ mergeload(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   return 0;
 }
 
-static int 
+static int
 mergestoredum(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   struct mergelocal *mergelocal;
@@ -426,7 +849,7 @@ mergestoredum(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **ar
   }
 }
 
-static int 
+static int
 mergeloaddum(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   char *s;
@@ -445,7 +868,7 @@ mergeloaddum(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **arg
   return 0;
 }
 
-static int 
+static int
 mergebbox(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   struct narray *array;
@@ -457,6 +880,7 @@ mergebbox(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   char *buf;
   struct GRAbbox bbox;
   int GC;
+  struct mergelocal *mergelocal;
 
   array=rval->array;
   if (arraynum(array)!=0) return 0;
@@ -473,7 +897,7 @@ mergebbox(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
     return 1;
   }
   if (strcmp(graf,buf)==0) newgra=TRUE;
-  else if (strcmp(" Ngraph GRA file",buf)==0) newgra=FALSE;
+  else if (strcmp(OLD_GRA_HEADER,buf)==0) newgra=FALSE;
   else {
     g_free(buf);
     fclose(fd);
@@ -499,8 +923,8 @@ mergebbox(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
       fclose(fd);
       return 1;
     }
-    if (newgra) rcode=GRAinput(GC,buf,lm,tm,zm);
-    else rcode=GRAinputold(GC,buf,lm,tm,zm);
+    if (newgra) rcode=GRAinput(GC,buf,0,0,10000);
+    else rcode=GRAinputold(GC,buf,0,0,10000);
     if (!rcode) {
       g_free(buf);
       fclose(fd);
@@ -521,36 +945,54 @@ mergebbox(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
     rval->array = NULL;
     return 1;
   }
+  _getobj(obj,"_local",inst,&mergelocal);
+  mergelocal->bbox[0] = bbox.minx;
+  mergelocal->bbox[1] = bbox.miny;
+  mergelocal->bbox[2] = bbox.maxx;
+  mergelocal->bbox[3] = bbox.maxy;
+  set_bbox(obj, inst, array, lm, tm, zm);
   rval->array=array;
   return 0;
 }
 
-static int 
+static int
 mergemove(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
-  int lm,tm;
+  int lm, tm, zm, x, y;
+  struct narray *array;
 
   _getobj(obj,"left_margin",inst,&lm);
   _getobj(obj,"top_margin",inst,&tm);
-  lm+=*(int *)argv[2];
-  tm+=*(int *)argv[3];
+  x = *(int *)argv[2];
+  y = *(int *)argv[3];
+  lm += x;
+  tm += y;
   if (_putobj(obj,"left_margin",inst,&lm)) return 1;
   if (_putobj(obj,"top_margin",inst,&tm)) return 1;
 
+#if 0
   if (clear_bbox(obj, inst))
     return 1;
+#else
+  _getobj(obj, "bbox", inst, &array);
+  if (arraynum(array) == 4) {
+    _getobj(obj, "zoom", inst, &zm);
+    set_bbox(obj, inst, array, lm, tm, zm);
+  }
+#endif
 
   return 0;
 }
 
 #define ZOOM_MAX 1000000
 
-static int 
+static int
 mergezoom(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   int lm,tm,zm;
   double zoom;
   int refx,refy;
+  struct narray *array;
 
   zoom=(*(int *)argv[2])/10000.0;
   refx=(*(int *)argv[3]);
@@ -570,13 +1012,20 @@ mergezoom(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   if (_putobj(obj,"top_margin",inst,&tm)) return 1;
   if (_putobj(obj,"zoom",inst,&zm)) return 1;
 
+#if 0
   if (clear_bbox(obj, inst))
     return 1;
+#else
+  _getobj(obj, "bbox", inst, &array);
+  if (arraynum(array) == 4) {
+    set_bbox(obj, inst, array, lm, tm, zm);
+  }
+#endif
 
   return 0;
 }
 
-static int 
+static int
 mergematch(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   int minx,miny,maxx,maxy,err;
@@ -613,11 +1062,13 @@ mergematch(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   return 0;
 }
 
-static int 
+static int
 mergegeometry(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
                  int argc,char **argv)
 {
   char *field;
+  int val;
+  struct narray *array;
 
   field=(char *)(argv[1]);
   if (strcmp(field,"zoom")==0) {
@@ -629,13 +1080,67 @@ mergegeometry(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
     }
   }
 
+#if 0
   if (clear_bbox(obj, inst)){
     return 1;
   }
+#else
+  val = * (int *) (argv[2]);
+  _getobj(obj, "bbox", inst, &array);
+  if (arraynum(array) == 4) {
+    int t, l, z;
+    _getobj(obj, "top_margin", inst, &t);
+    _getobj(obj, "left_margin", inst, &l);
+    _getobj(obj, "zoom", inst, &z);
+    switch (field[0]) {
+    case 't':
+      t = val;
+      break;
+    case 'l':
+      l = val;
+      break;
+    case 'z':
+      z = val;
+      break;
+    }
+    set_bbox(obj, inst, array, l, t, z);
+  }
+#endif
 
   return 0;
 }
 
+static int
+merge_inst_dup(struct objlist *obj, N_VALUE *src, N_VALUE *dest)
+{
+  int i;
+
+  i = obj_get_field_pos(obj, "_local");
+  if (i < 0) {
+    return 1;
+  }
+  dest[i].ptr = g_memdup(src[i].ptr, sizeof(struct mergelocal));
+  /*   mergelocal->storefd may be NULL */
+  if (dest[i].ptr == NULL) {
+    return 1;
+  }
+  return 0;
+}
+
+static int
+merge_inst_free(struct objlist *obj, N_VALUE *inst)
+{
+  int i;
+
+  i = obj_get_field_pos(obj, "_local");
+  if (i == -1) {
+      return 1;
+  }
+  g_free(inst[i].ptr);
+  /* mergelocal->storefd may be NULL */
+  return 0;
+}
+
 static struct objtable merge[] = {
   {"init",NVFUNC,NEXEC,mergeinit,NULL,0},
   {"done",NVFUNC,NEXEC,mergedone,NULL,0},
@@ -673,5 +1178,8 @@ void *
 addmerge(void)
 /* addmerge() returns NULL on error */
 {
-  return addobject(NAME,NULL,PARENT,OVERSION,TBLNUM,merge,ERRNUM,mergeerrorlist,NULL,NULL);
+  struct objlist *merge_obj;
+  merge_obj = addobject(NAME,NULL,PARENT,OVERSION,TBLNUM,merge,ERRNUM,mergeerrorlist,NULL,NULL);
+  obj_set_undo_func(merge_obj, merge_inst_dup, merge_inst_free);
+  return merge_obj;
 }
