@@ -1,24 +1,24 @@
-/* 
+/*
  * $Id: oline.c,v 1.13 2010-03-04 08:30:16 hito Exp $
- * 
+ *
  * This file is part of "Ngraph for X11".
- * 
+ *
  * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
- * 
+ *
  * "Ngraph for X11" is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * "Ngraph for X11" is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- * 
+ *
  */
 
 #include "common.h"
@@ -83,9 +83,9 @@ static char *path_type[]={
   NULL,
 };
 
-static int 
+static int
 arrowinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
-{  
+{
   int width, headlen, headwidth, miter, stroke, join, prm, type, alpha;
   struct narray *expand_points;
 
@@ -144,7 +144,7 @@ arrowinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   return 0;
 }
 
-static int 
+static int
 arrowdone(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   if (_exeparent(obj, argv[1], inst, rval, argc, argv)) return 1;
@@ -154,7 +154,7 @@ arrowdone(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
 #define ARROW_SIZE_MIN 10000
 #define ARROW_SIZE_MAX 200000
 
-static int 
+static int
 arrowput(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   char *field;
@@ -189,230 +189,23 @@ arrowput(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
 }
 
 static int
-curve_expand(double c[], double x0, double y0, diffunc gdiff, intpfunc gintpf, struct narray *expand_points)
+opath_curve_expand_points(struct objlist *obj, N_VALUE *inst, int intp, struct narray *expand_points)
 {
-  double d, dx, dy, ddx, ddy, dd, x, y;
-  int gx2, gy2;
-
-  d = 0;
-  while (d < 1) {
-    gdiff(d, c, &dx, &dy, &ddx, &ddy, NULL);
-    if (fabs(dx) + fabs(ddx) / 3 <= 1E-100) {
-      dx = 1;
-    } else {
-      dx = sqrt(fabs(2 / (fabs(dx) + fabs(ddx) / 3)));
-    }
-
-    if (fabs(dy) + fabs(ddy) / 3 <= 1E-100) {
-      dy = 1;
-    } else {
-      dy = sqrt(fabs(2 / (fabs(dy) + fabs(ddy) / 3)));
-    }
-
-    dd = (dx < dy) ? dx : dy;
-    d += dd;
-
-    if (d > 1) {
-      d = 1;
-    }
-
-    gintpf(d, c, x0, y0, &x, &y, NULL);
-
-    gx2 = nround(x);
-    gy2 = nround(y);
-
-    arrayadd(expand_points, &gx2);
-    arrayadd(expand_points, &gy2);
-  }
-
-  return TRUE;
-}
-
-static int
-curve_expand_points(struct objlist *obj, N_VALUE *inst, int intp, struct narray *expand_points)
-{
-  int i, j, num, bsize, spcond, x, y;
+  int num, r;
   struct narray *points;
-  double c[8];
-  double *buf;
-  double bs1[7], bs2[7], bs3[4], bs4[4];
   int *pdata;
 
   _getobj(obj, "points", inst, &points);
-
-  num = arraynum(points)/2;
+  num = arraynum(points) / 2;
   pdata = arraydata(points);
-
-  arrayclear(expand_points);
-
-  switch (intp) {
-  case INTERPOLATION_TYPE_SPLINE:
-  case INTERPOLATION_TYPE_SPLINE_CLOSE:
-    if (num < 2)
-      return 0;
-
-    bsize = num + 1;
-    buf = g_malloc(sizeof(double) * 9 * bsize);
-    if (buf == NULL)
-      return 1;
-
-    for (i = 0; i < bsize; i++)
-      buf[i]=i;
-
-    for (i = 0; i < num; i++) {
-      buf[bsize + i] = pdata[i * 2];
-      buf[bsize * 2 + i] = pdata[i * 2 + 1];
-    }
-
-    if (intp == INTERPOLATION_TYPE_SPLINE) {
-      spcond = SPLCND2NDDIF;
-    } else {
-      spcond = SPLCNDPERIODIC;
-      if (buf[num - 1 + bsize] != buf[bsize] || buf[num - 1 + 2 * bsize] != buf[2 * bsize]) {
-        buf[num + bsize] = buf[bsize];
-        buf[num + 2 * bsize] = buf[2 * bsize];
-        num++;
-      }
-    }
-
-    if (spline(buf,
-	       buf + bsize,
-	       buf + 3 * bsize,
-	       buf + 4 * bsize,
-	       buf + 5 * bsize,
-	       num, spcond, spcond, 0, 0)) {
-      g_free(buf);
-      error(obj, ERRSPL);
-      return 1;
-    }
-
-    if (spline(buf,
-	       buf + 2 * bsize,
-	       buf + 6 * bsize,
-	       buf + 7 * bsize,
-	       buf + 8 * bsize,
-	       num, spcond, spcond, 0, 0)) {
-      g_free(buf);
-      error(obj, ERRSPL);
-      return 1;
-    }
-
-    x = nround(buf[bsize]);
-    y = nround(buf[bsize * 2]);
-    arrayadd(expand_points, &x);
-    arrayadd(expand_points, &y);
-    for (i = 0; i < num - 1; i++) {
-      for (j = 0; j < 6; j++) {
-	c[j] = buf[i + (j + 3) * bsize];
-      }
-
-      if (! curve_expand(c, buf[i + bsize], buf[i + 2 * bsize], splinedif, splineint, expand_points)) {
-        break;
-      }
-    }
-    g_free(buf);
-    break;
-  case INTERPOLATION_TYPE_BSPLINE:
-    if (num < 7) {
-      return 0;
-    }
-
-    for (i = 0; i < 7; i++) {
-      bs1[i] = pdata[i * 2];
-      bs2[i] = pdata[i * 2 + 1];
-    }
-
-    for (j = 0; j < 2; j++) {
-      bspline(j + 1, bs1 + j, c);
-      bspline(j + 1, bs2 + j, c + 4);
-
-      if (j == 0) {
-	x = nround(c[0]);
-	y = nround(c[4]);
-	arrayadd(expand_points, &x);
-	arrayadd(expand_points, &y);
-      }
-
-      if (! curve_expand(c, c[0], c[4], bsplinedif, bsplineint, expand_points)) {
-	return 0;
-      }
-    }
-
-    for (; i < num; i++) {
-      for (j = 1; j < 7; j++) {
-        bs1[j - 1] = bs1[j];
-        bs2[j - 1] = bs2[j];
-      }
-      bs1[6] = pdata[i*2];
-      bs2[6] = pdata[i*2+1];
-      bspline(0, bs1 + 1, c);
-      bspline(0, bs2 + 1, c + 4);
-      if (! curve_expand(c, c[0], c[4], bsplinedif, bsplineint, expand_points)) {
-	return 0;
-      }
-    }
-
-    for (j = 0; j < 2; j++) {
-      bspline(j + 3, bs1 + j + 2, c);
-      bspline(j + 3, bs2 + j + 2, c + 4);
-      if (! curve_expand(c, c[0], c[4], bsplinedif, bsplineint, expand_points)) {
-	return 0;
-      }
-    }
-    break;
-  case INTERPOLATION_TYPE_BSPLINE_CLOSE:
-    if (num < 4) {
-      return 0;
-    }
-
-    for (i = 0; i < 4; i++) {
-      bs1[i] = pdata[i * 2];
-      bs3[i] = pdata[i * 2];
-      bs2[i] = pdata[i * 2 + 1];
-      bs4[i] = pdata[i * 2 + 1];
-    }
-
-    bspline(0, bs1, c);
-    bspline(0, bs2, c + 4);
-    x = nround(c[0]);
-    y = nround(c[4]);
-    arrayadd(expand_points, &x);
-    arrayadd(expand_points, &y);
-
-    if (! curve_expand(c, c[0], c[4], bsplinedif, bsplineint, expand_points)) {
-      return 0;
-    }
-
-    for (; i < num; i++) {
-      for (j = 1; j < 4; j++) {
-        bs1[j - 1] = bs1[j];
-        bs2[j - 1] = bs2[j];
-      }
-
-      bs1[3] = pdata[i * 2];
-      bs2[3] = pdata[i * 2 + 1];
-      bspline(0, bs1, c);
-      bspline(0, bs2, c + 4);
-      if (! curve_expand(c, c[0], c[4], bsplinedif, bsplineint, expand_points)) {
-	return 0;
-      }
-    }
-    for (j = 0; j < 3; j++) {
-      bs1[4 + j] = bs3[j];
-      bs2[4 + j] = bs4[j];
-      bspline(0, bs1 + j + 1, c);
-      bspline(0, bs2 + j + 1, c + 4);
-      if (! curve_expand(c, c[0], c[4], bsplinedif, bsplineint, expand_points)) {
-	return 0;
-      }
-    }
-    break;
+  r = curve_expand_points(pdata, num, intp, expand_points);
+  if (r) {
+    error(obj, ERRSPL);
   }
-
-  return 0;
+  return r;
 }
 
-static void 
+static void
 curve_clear(struct objlist *obj,N_VALUE *inst)
 {
   struct narray *expand_points;
@@ -559,7 +352,7 @@ draw_fill(struct objlist *obj, N_VALUE *inst, int GC, int *points2, int num)
   GRAdrawpoly(GC, num, points2, fill_rule + 1);
 }
 
-static int 
+static int
 arrowdraw(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   int GC, w, h, intp, i, j, num, close_path;
@@ -590,7 +383,7 @@ arrowdraw(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
     _getobj(obj, "interpolation", inst, &intp);
     _getobj(obj, "_points",       inst, &points);
     if (arraynum(points) == 0) {
-      curve_expand_points(obj, inst, intp, points);
+      opath_curve_expand_points(obj, inst, intp, points);
     }
     if (intp == INTERPOLATION_TYPE_SPLINE_CLOSE ||
 	intp == INTERPOLATION_TYPE_BSPLINE_CLOSE) {
@@ -642,7 +435,7 @@ arrowdraw(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   return 0;
 }
 
-static int 
+static int
 arrowbbox(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   int minx, miny, maxx, maxy;
@@ -674,7 +467,7 @@ arrowbbox(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
     _getobj(obj, "interpolation", inst, &intp);
     _getobj(obj, "_points",       inst, &points);
     if (arraynum(points) == 0) {
-      curve_expand_points(obj, inst, intp, points);
+      opath_curve_expand_points(obj, inst, intp, points);
     }
   } else {
     _getobj(obj, "points", inst, &points);
@@ -801,7 +594,7 @@ arrowbbox(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   return 0;
 }
 
-static int 
+static int
 set_points(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   curve_clear(obj, inst);
@@ -861,7 +654,7 @@ point_match(struct objlist *obj, N_VALUE *inst, int type, int fill, int err, int
     _getobj(obj, "interpolation", inst, &intp);
     _getobj(obj, "_points", inst, &points);
     if (arraynum(points) == 0) {
-      curve_expand_points(obj, inst, intp, points);
+      opath_curve_expand_points(obj, inst, intp, points);
     }
   } else {
     _getobj(obj, "points", inst, &points);
@@ -893,7 +686,7 @@ point_match(struct objlist *obj, N_VALUE *inst, int type, int fill, int err, int
   return r;
 }
 
-static int 
+static int
 curvematch(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   int minx, miny, maxx, maxy, err;
@@ -940,9 +733,9 @@ curvematch(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
     bminy = arraynget_int(array, 1);
     bmaxx = arraynget_int(array, 2);
     bmaxy = arraynget_int(array, 3);
-    if (minx <= bminx && bminx <= maxx && 
-	minx <= bmaxx && bmaxx <= maxx && 
-	miny <= bminy && bminy <= maxy && 
+    if (minx <= bminx && bminx <= maxx &&
+	minx <= bmaxx && bmaxx <= maxx &&
+	miny <= bminy && bminy <= maxy &&
 	miny <= bmaxy && bmaxy <= maxy) {
       rval->i = TRUE;
     }
@@ -950,7 +743,7 @@ curvematch(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
   return 0;
 }
 
-static int 
+static int
 curve_flip(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   curve_clear(obj, inst);
@@ -958,7 +751,7 @@ curve_flip(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
   return legendflip(obj, inst, rval, argc, argv);
 }
 
-static int 
+static int
 curve_move(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   struct narray *points;
@@ -981,7 +774,7 @@ curve_move(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
   return legendmove(obj, inst, rval, argc, argv);
 }
 
-static int 
+static int
 curve_rotate(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   curve_clear(obj, inst);
@@ -989,7 +782,7 @@ curve_rotate(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char *
   return legendrotate(obj, inst, rval, argc, argv);
 }
 
-static int 
+static int
 curve_zoom(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   curve_clear(obj, inst);
@@ -997,7 +790,7 @@ curve_zoom(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
   return legendzoom(obj, inst, rval, argc, argv);
 }
 
-static int 
+static int
 curve_change(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   curve_clear(obj, inst);
@@ -1005,7 +798,7 @@ curve_change(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char *
   return legendchange(obj, inst, rval, argc, argv);
 }
 
-static int 
+static int
 put_fill_mode(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   int mode, rule;
